Package: tinycodetfakepkg5
Title: What the Package Does (One Line, Title Case)
Version: 0.0.0.9000
Authors@R: 
    person("First", "Last", , "first.last@example.com", role = c("aut", "cre"),
           comment = c(ORCID = "YOUR-ORCID-ID"))
Description: What the package does (one paragraph).
License: `use_mit_license()`, `use_gpl3_license()` or friends to pick a
        license
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-12-06 12:21:16 UTC; Tony
Author: First Last [aut, cre] (YOUR-ORCID-ID)
Maintainer: First Last <first.last@example.com>
Built: R 4.3.2; ; 2023-12-06 12:21:16 UTC; windows
