Package: tinycodetfakepkg1
Title: What the Package Does (One Line, Title Case)
Version: 1.0
Authors@R: 
    person("First", "Last", , "first.last@example.com", role = c("aut", "cre"),
           comment = c(ORCID = "YOUR-ORCID-ID"))
Description: What the package does (one paragraph).
License: `use_mit_license()`, `use_gpl3_license()` or friends to pick a
        license
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
Enhances: tinycodetfakepkg3
NeedsCompilation: no
Packaged: 2024-07-20 19:32:56 UTC; Tony
Author: First Last [aut, cre] (YOUR-ORCID-ID)
Maintainer: First Last <first.last@example.com>
Built: R 4.4.0; ; 2024-07-20 19:32:57 UTC; windows
