Package: tinycodet
Title: Functions to Help in your Coding Etiquette
Version: 0.7.0
Authors@R: person("Tony", "Wilkes", role = c("aut", "cre", "cph"),
                  email = "tony_a_wilkes@outlook.com",
                  comment = c(ORCID = "0000-0001-9498-8379"))
Description: Adds some functions to help in your coding etiquette.
    'tinycodet' primarily focuses on 4 aspects.
    1) Safer decimal (in)equality testing,
    standard-evaluated alternatives to with() and aes(),
    and other functions for safer coding.
    2) A new package import system,
    that attempts to combine the benefits of using a package without attaching it,
    with the benefits of attaching a package.
    3) Extending the string manipulation capabilities of the 'stringi' R package.
    4) Reducing repetitive code.
    Besides linking to 'Rcpp', 'tinycodet' has only one other dependency, namely 'stringi'.
License: MIT + file LICENSE
Encoding: UTF-8
LinkingTo: Rcpp
RoxygenNote: 7.3.2
Suggests: tinytest, ggplot2, mgcv, nlme, collapse, kit, knitr,
        rmarkdown, roxygen2
Depends: R (>= 4.1.0)
Imports: methods, Rcpp (>= 1.0.11), stringi (>= 1.7.12)
URL: https://github.com/tony-aw/tinycodet/,
        https://tony-aw.github.io/tinycodet/
BugReports: https://github.com/tony-aw/tinycodet/issues/
Language: en-gb
NeedsCompilation: yes
Packaged: 2026-06-07 16:23:50 UTC; Tony
Author: Tony Wilkes [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-9498-8379>)
Maintainer: Tony Wilkes <tony_a_wilkes@outlook.com>
Repository: CRAN
Date/Publication: 2026-06-08 20:10:03 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-06-08 23:05:14 UTC; unix
Archs: tinycodet.so.dSYM
