# Generated by roxygen2: do not edit by hand

export("%<-c%")
export("%col~%")
export("%d!=%")
export("%d!{}%")
export("%d<%")
export("%d<=%")
export("%d==%")
export("%d>%")
export("%d>=%")
export("%d{}%")
export("%installed in%")
export("%row~%")
export("%s!{}%")
export("%s$%")
export("%s*%")
export("%s+%")
export("%s-%")
export("%s/%")
export("%s//%")
export("%s<>%")
export("%s><%")
export("%sget%")
export("%ss%")
export("%strim%")
export("%s{}%")
export("strfind<-")
export(aes_pro)
export(attr.import)
export(help.import)
export(import_LL)
export(import_as)
export(import_data)
export(import_inops)
export(import_inops.control)
export(import_int)
export(is.tinyimport)
export(is_wholenumber)
export(lock_TF)
export(pkg_get_deps)
export(pkg_get_deps_minimal)
export(pkg_lsf)
export(pversion_check4mismatch)
export(pversion_report)
export(report_inops)
export(s_chrcls)
export(s_coll)
export(s_fixed)
export(s_regex)
export(safer_partialmatch)
export(source_selection)
export(strcut_brk)
export(strcut_loc)
export(strfind)
export(stri_c_mat)
export(stri_join_mat)
export(stri_locate_ith)
export(stri_locate_ith_boundaries)
export(stri_locate_ith_charclass)
export(stri_locate_ith_coll)
export(stri_locate_ith_fixed)
export(stri_locate_ith_regex)
export(stri_paste_mat)
export(transform_if)
export(with_pro)
importFrom(Rcpp,evalCpp)
importFrom(stringi,"%s$%")
importFrom(stringi,"%s*%")
importFrom(stringi,"%s+%")
useDynLib(tinycodet, .registration=TRUE)
