# Generated by roxygen2: do not edit by hand

export("%op31%")
export("%op32%")
export("%opover%")
export(acf)
export(fun11)
export(fun31)
export(fun32)
export(fun_overwritten)
importFrom(stats,acf)
importFrom(tinycodetfakepkg1,fun11)
